#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/wdt.h>
#ifndef F_CPU
#define F_CPU 8000000UL
#warning F_CPU not defined
#endif
//Checksumme für die Datenübertragung zum Taschenrechner
uint8_t checksum;
#include "uart.cpp"
#include "rf12.cpp"
//Datenspeicher
//Liste1:0-249
//Liste2:250-499
//Liste3:500-749
//Liste4:750-999
//Liste5:1000-1249
//1250-1509: Variablen A-Z
uint8_t data[1520];
uint8_t list_length[5]={1,1,1,1,1};//Speicher für die Anzahl der Elemente in den Listen
uint8_t rfm_buffer[252];//speichert ankommende Daten der Funkverbindung
uint8_t rfm_receive=0; //Anzahl der zu empfangenden Bytes
uint8_t rfm_count=0;//zählt mit, wie viele Bytes über Funk empfangen wurden
uint8_t rfm_vars=0;//Anzahl der Variablen im Taschenrechnerformat, die gesendet wurden
volatile uint8_t count=0;
#include "funk.cpp"
//wartet X ms lang
void long_delay(uint16_t ms)
{
	for(;ms>0;ms--)_delay_ms(1);
}
#include "gtr_routines.cpp"
int main(void)
{
	wdt_disable();
	//USART wird initialisiert
	uart_init();
	//kurze Pause, damit das RFM12 sicher gestartet ist
	long_delay(50);
	//Initialisierung des RFM12
	RFM12B_init_hard();
	RFM12B_init_soft();
	//Timer 0 mit Prescaler 8 starten (erzeugt jede ms einen Overflow)
	TCCR0 |= (1<<CS01);
	//Timer 2 mit Prescaler 1024 starten
	TCCR2 |= (1<<CS22) | (1<<CS21) | (1<<CS20);
	//Timer-Overflow-Interrupts aktivieren
	TIMSK |= (1<<TOIE0) | (1<<TOIE2);
	MCUCR |= (1<<ISC11) | (1<<ISC10);
	GICR |= (1<<INT1);
	sei();
	data[1500]=1;
	data[1508]=1;
	data[1509]=11;
	//Endlosschleife für die Kommunikation mit dem Taschenrechner (Funkverbindung läuft im Interrupt)
	wdt_enable(WDTO_2S);
	while(1)
	{
		//wartet bis der Taschenrechner die Übertragung beginnt
		while(uart_getc()!=0x15)wdt_reset();
		//Funktion, die den Datenverkehr mit dem Taschenrechner steuert aufrufen
		gtr();
	}
}
//Timer0-Overflow-Interrupt:
//wird jede ms aufgerufen
//überprüft, ob ein neues Byte beim RFM12 ankam und wertet dieses aus
//Zähler für die Funkverbindung wird zurückgesetzt
ISR(TIMER0_OVF_vect)
{
	TCNT0 = 127;
	if(RFM12B_ffit())
	{
		TCNT2 = 0;
		rfm_data();
	}
}
//Timer2-Overflow-Interrupt
//Funktionsweise:
//sobald Daten über Funk empfangen werden, wird der Zählerstand auf 0 gesetzt
//es dürfen erst Daten gesendet werden, wenn der Zählerstand größer als 100 ist
//(erst 50 ms nach dem letzten Funkempfang dürfen Daten gesendet werden. Dadurch soll
//verhindert werden, dass die Funkmodule sich gegenseitig stören).
ISR(TIMER2_OVF_vect)
{
	TCNT2 = 100;
}

